// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "ui/UIElement.hpp"

#define UI_ALIGN_AUTO -1

namespace Dawn {
  enum class UIAlignmentType {
    START,
    MIDDLE,
    END,
    STRETCH
  };

  enum class UIAlignmentUnit {
    PIXEL,
    SCALE,
    PERCENT,
    RATIO
  };

  class UIAlignableElement : public UIElement {
    protected:
      glm::vec2 position;
      glm::vec2 size;
      
      /**
       * Updates the alignment of this element ONLY.
       * 
       * @param parentPosition The position of the parent.
       * @param parentSize The size of the parent.
       * @param canvasScale The scale of the canvas.
       */
      virtual void updateSelfAlignment(
        const glm::vec2 parentPosition,
        const glm::vec2 parentSize,
        const float_t canvasScale
      );

    public:
      // Primary alignment controls
      glm::vec4 align = glm::vec4(0, 0, UI_ALIGN_AUTO, UI_ALIGN_AUTO);
      enum UIAlignmentType alignX = UIAlignmentType::START;
      enum UIAlignmentType alignY = UIAlignmentType::START;
      enum UIAlignmentUnit alignUnit[4];
      
      std::function<
        void(const glm::vec2, const glm::vec2)
      > eventAlignmentUpdated;

      /**
       * Constructor for the UIAlignableElement.
       */
      UIAlignableElement();

      float_t getSelfWidth() override;
      float_t getSelfHeight() override;
      void updateAlignment(
        const glm::vec2 parentPosition,
        const glm::vec2 parentSize,
        const float_t canvasScale
      ) override;
  };
}